// Generated by the protocol buffer compiler.  DO NOT EDIT!
// source: protobuf-c/protobuf-c.proto

#ifndef GOOGLE_PROTOBUF_INCLUDED_protobuf_2dc_2fprotobuf_2dc_2eproto
#define GOOGLE_PROTOBUF_INCLUDED_protobuf_2dc_2fprotobuf_2dc_2eproto

#include <limits>
#include <string>

#include <google/protobuf/port_def.inc>
#if PROTOBUF_VERSION < 3021000
#error This file was generated by a newer version of protoc which is
#error incompatible with your Protocol Buffer headers. Please update
#error your headers.
#endif
#if 3021005 < PROTOBUF_MIN_PROTOC_VERSION
#error This file was generated by an older version of protoc which is
#error incompatible with your Protocol Buffer headers. Please
#error regenerate this file with a newer version of protoc.
#endif

#include <google/protobuf/port_undef.inc>
#include <google/protobuf/io/coded_stream.h>
#include <google/protobuf/arena.h>
#include <google/protobuf/arenastring.h>
#include <google/protobuf/generated_message_util.h>
#include <google/protobuf/metadata_lite.h>
#include <google/protobuf/generated_message_reflection.h>
#include <google/protobuf/message.h>
#include <google/protobuf/repeated_field.h>  // IWYU pragma: export
#include <google/protobuf/extension_set.h>  // IWYU pragma: export
#include <google/protobuf/unknown_field_set.h>
#include <google/protobuf/descriptor.pb.h>
// @@protoc_insertion_point(includes)
#include <google/protobuf/port_def.inc>
#define PROTOBUF_INTERNAL_EXPORT_protobuf_2dc_2fprotobuf_2dc_2eproto
PROTOBUF_NAMESPACE_OPEN
namespace internal {
class AnyMetadata;
}  // namespace internal
PROTOBUF_NAMESPACE_CLOSE

// Internal implementation detail -- do not use these members.
struct TableStruct_protobuf_2dc_2fprotobuf_2dc_2eproto {
  static const uint32_t offsets[];
};
extern const ::PROTOBUF_NAMESPACE_ID::internal::DescriptorTable descriptor_table_protobuf_2dc_2fprotobuf_2dc_2eproto;
class ProtobufCFieldOptions;
struct ProtobufCFieldOptionsDefaultTypeInternal;
extern ProtobufCFieldOptionsDefaultTypeInternal _ProtobufCFieldOptions_default_instance_;
class ProtobufCFileOptions;
struct ProtobufCFileOptionsDefaultTypeInternal;
extern ProtobufCFileOptionsDefaultTypeInternal _ProtobufCFileOptions_default_instance_;
class ProtobufCMessageOptions;
struct ProtobufCMessageOptionsDefaultTypeInternal;
extern ProtobufCMessageOptionsDefaultTypeInternal _ProtobufCMessageOptions_default_instance_;
PROTOBUF_NAMESPACE_OPEN
template<> ::ProtobufCFieldOptions* Arena::CreateMaybeMessage<::ProtobufCFieldOptions>(Arena*);
template<> ::ProtobufCFileOptions* Arena::CreateMaybeMessage<::ProtobufCFileOptions>(Arena*);
template<> ::ProtobufCMessageOptions* Arena::CreateMaybeMessage<::ProtobufCMessageOptions>(Arena*);
PROTOBUF_NAMESPACE_CLOSE

// ===================================================================

class ProtobufCFileOptions final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:ProtobufCFileOptions) */ {
 public:
  inline ProtobufCFileOptions() : ProtobufCFileOptions(nullptr) {}
  ~ProtobufCFileOptions() override;
  explicit PROTOBUF_CONSTEXPR ProtobufCFileOptions(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  ProtobufCFileOptions(const ProtobufCFileOptions& from);
  ProtobufCFileOptions(ProtobufCFileOptions&& from) noexcept
    : ProtobufCFileOptions() {
    *this = ::std::move(from);
  }

  inline ProtobufCFileOptions& operator=(const ProtobufCFileOptions& from) {
    CopyFrom(from);
    return *this;
  }
  inline ProtobufCFileOptions& operator=(ProtobufCFileOptions&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet& unknown_fields() const {
    return _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance);
  }
  inline ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet* mutable_unknown_fields() {
    return _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const ProtobufCFileOptions& default_instance() {
    return *internal_default_instance();
  }
  static inline const ProtobufCFileOptions* internal_default_instance() {
    return reinterpret_cast<const ProtobufCFileOptions*>(
               &_ProtobufCFileOptions_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    0;

  friend void swap(ProtobufCFileOptions& a, ProtobufCFileOptions& b) {
    a.Swap(&b);
  }
  inline void Swap(ProtobufCFileOptions* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ProtobufCFileOptions* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  ProtobufCFileOptions* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<ProtobufCFileOptions>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const ProtobufCFileOptions& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom( const ProtobufCFileOptions& from) {
    ProtobufCFileOptions::MergeImpl(*this, from);
  }
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::PROTOBUF_NAMESPACE_ID::Arena* arena, bool is_message_owned);
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(ProtobufCFileOptions* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "ProtobufCFileOptions";
  }
  protected:
  explicit ProtobufCFileOptions(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kCPackageFieldNumber = 6,
    kNoGenerateFieldNumber = 1,
    kConstStringsFieldNumber = 4,
    kUseOneofFieldNameFieldNumber = 5,
    kGenPackHelpersFieldNumber = 2,
    kGenInitHelpersFieldNumber = 3,
  };
  // optional string c_package = 6;
  bool has_c_package() const;
  private:
  bool _internal_has_c_package() const;
  public:
  void clear_c_package();
  const std::string& c_package() const;
  template <typename ArgT0 = const std::string&, typename... ArgT>
  void set_c_package(ArgT0&& arg0, ArgT... args);
  std::string* mutable_c_package();
  PROTOBUF_NODISCARD std::string* release_c_package();
  void set_allocated_c_package(std::string* c_package);
  private:
  const std::string& _internal_c_package() const;
  inline PROTOBUF_ALWAYS_INLINE void _internal_set_c_package(const std::string& value);
  std::string* _internal_mutable_c_package();
  public:

  // optional bool no_generate = 1 [default = false];
  bool has_no_generate() const;
  private:
  bool _internal_has_no_generate() const;
  public:
  void clear_no_generate();
  bool no_generate() const;
  void set_no_generate(bool value);
  private:
  bool _internal_no_generate() const;
  void _internal_set_no_generate(bool value);
  public:

  // optional bool const_strings = 4 [default = false];
  bool has_const_strings() const;
  private:
  bool _internal_has_const_strings() const;
  public:
  void clear_const_strings();
  bool const_strings() const;
  void set_const_strings(bool value);
  private:
  bool _internal_const_strings() const;
  void _internal_set_const_strings(bool value);
  public:

  // optional bool use_oneof_field_name = 5 [default = false];
  bool has_use_oneof_field_name() const;
  private:
  bool _internal_has_use_oneof_field_name() const;
  public:
  void clear_use_oneof_field_name();
  bool use_oneof_field_name() const;
  void set_use_oneof_field_name(bool value);
  private:
  bool _internal_use_oneof_field_name() const;
  void _internal_set_use_oneof_field_name(bool value);
  public:

  // optional bool gen_pack_helpers = 2 [default = true];
  bool has_gen_pack_helpers() const;
  private:
  bool _internal_has_gen_pack_helpers() const;
  public:
  void clear_gen_pack_helpers();
  bool gen_pack_helpers() const;
  void set_gen_pack_helpers(bool value);
  private:
  bool _internal_gen_pack_helpers() const;
  void _internal_set_gen_pack_helpers(bool value);
  public:

  // optional bool gen_init_helpers = 3 [default = true];
  bool has_gen_init_helpers() const;
  private:
  bool _internal_has_gen_init_helpers() const;
  public:
  void clear_gen_init_helpers();
  bool gen_init_helpers() const;
  void set_gen_init_helpers(bool value);
  private:
  bool _internal_gen_init_helpers() const;
  void _internal_set_gen_init_helpers(bool value);
  public:

  // @@protoc_insertion_point(class_scope:ProtobufCFileOptions)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
    ::PROTOBUF_NAMESPACE_ID::internal::HasBits<1> _has_bits_;
    mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
    ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr c_package_;
    bool no_generate_;
    bool const_strings_;
    bool use_oneof_field_name_;
    bool gen_pack_helpers_;
    bool gen_init_helpers_;
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_protobuf_2dc_2fprotobuf_2dc_2eproto;
};
// -------------------------------------------------------------------

class ProtobufCMessageOptions final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:ProtobufCMessageOptions) */ {
 public:
  inline ProtobufCMessageOptions() : ProtobufCMessageOptions(nullptr) {}
  ~ProtobufCMessageOptions() override;
  explicit PROTOBUF_CONSTEXPR ProtobufCMessageOptions(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  ProtobufCMessageOptions(const ProtobufCMessageOptions& from);
  ProtobufCMessageOptions(ProtobufCMessageOptions&& from) noexcept
    : ProtobufCMessageOptions() {
    *this = ::std::move(from);
  }

  inline ProtobufCMessageOptions& operator=(const ProtobufCMessageOptions& from) {
    CopyFrom(from);
    return *this;
  }
  inline ProtobufCMessageOptions& operator=(ProtobufCMessageOptions&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet& unknown_fields() const {
    return _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance);
  }
  inline ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet* mutable_unknown_fields() {
    return _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const ProtobufCMessageOptions& default_instance() {
    return *internal_default_instance();
  }
  static inline const ProtobufCMessageOptions* internal_default_instance() {
    return reinterpret_cast<const ProtobufCMessageOptions*>(
               &_ProtobufCMessageOptions_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    1;

  friend void swap(ProtobufCMessageOptions& a, ProtobufCMessageOptions& b) {
    a.Swap(&b);
  }
  inline void Swap(ProtobufCMessageOptions* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ProtobufCMessageOptions* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  ProtobufCMessageOptions* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<ProtobufCMessageOptions>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const ProtobufCMessageOptions& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom( const ProtobufCMessageOptions& from) {
    ProtobufCMessageOptions::MergeImpl(*this, from);
  }
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::PROTOBUF_NAMESPACE_ID::Arena* arena, bool is_message_owned);
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(ProtobufCMessageOptions* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "ProtobufCMessageOptions";
  }
  protected:
  explicit ProtobufCMessageOptions(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kBaseFieldNameFieldNumber = 3,
    kGenPackHelpersFieldNumber = 1,
    kGenInitHelpersFieldNumber = 2,
  };
  // optional string base_field_name = 3 [default = "base"];
  bool has_base_field_name() const;
  private:
  bool _internal_has_base_field_name() const;
  public:
  void clear_base_field_name();
  const std::string& base_field_name() const;
  template <typename ArgT0 = const std::string&, typename... ArgT>
  void set_base_field_name(ArgT0&& arg0, ArgT... args);
  std::string* mutable_base_field_name();
  PROTOBUF_NODISCARD std::string* release_base_field_name();
  void set_allocated_base_field_name(std::string* base_field_name);
  private:
  const std::string& _internal_base_field_name() const;
  inline PROTOBUF_ALWAYS_INLINE void _internal_set_base_field_name(const std::string& value);
  std::string* _internal_mutable_base_field_name();
  public:

  // optional bool gen_pack_helpers = 1 [default = false];
  bool has_gen_pack_helpers() const;
  private:
  bool _internal_has_gen_pack_helpers() const;
  public:
  void clear_gen_pack_helpers();
  bool gen_pack_helpers() const;
  void set_gen_pack_helpers(bool value);
  private:
  bool _internal_gen_pack_helpers() const;
  void _internal_set_gen_pack_helpers(bool value);
  public:

  // optional bool gen_init_helpers = 2 [default = true];
  bool has_gen_init_helpers() const;
  private:
  bool _internal_has_gen_init_helpers() const;
  public:
  void clear_gen_init_helpers();
  bool gen_init_helpers() const;
  void set_gen_init_helpers(bool value);
  private:
  bool _internal_gen_init_helpers() const;
  void _internal_set_gen_init_helpers(bool value);
  public:

  // @@protoc_insertion_point(class_scope:ProtobufCMessageOptions)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
    ::PROTOBUF_NAMESPACE_ID::internal::HasBits<1> _has_bits_;
    mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
    static const ::PROTOBUF_NAMESPACE_ID::internal::LazyString _i_give_permission_to_break_this_code_default_base_field_name_;
    ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr base_field_name_;
    bool gen_pack_helpers_;
    bool gen_init_helpers_;
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_protobuf_2dc_2fprotobuf_2dc_2eproto;
};
// -------------------------------------------------------------------

class ProtobufCFieldOptions final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:ProtobufCFieldOptions) */ {
 public:
  inline ProtobufCFieldOptions() : ProtobufCFieldOptions(nullptr) {}
  ~ProtobufCFieldOptions() override;
  explicit PROTOBUF_CONSTEXPR ProtobufCFieldOptions(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  ProtobufCFieldOptions(const ProtobufCFieldOptions& from);
  ProtobufCFieldOptions(ProtobufCFieldOptions&& from) noexcept
    : ProtobufCFieldOptions() {
    *this = ::std::move(from);
  }

  inline ProtobufCFieldOptions& operator=(const ProtobufCFieldOptions& from) {
    CopyFrom(from);
    return *this;
  }
  inline ProtobufCFieldOptions& operator=(ProtobufCFieldOptions&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet& unknown_fields() const {
    return _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance);
  }
  inline ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet* mutable_unknown_fields() {
    return _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const ProtobufCFieldOptions& default_instance() {
    return *internal_default_instance();
  }
  static inline const ProtobufCFieldOptions* internal_default_instance() {
    return reinterpret_cast<const ProtobufCFieldOptions*>(
               &_ProtobufCFieldOptions_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    2;

  friend void swap(ProtobufCFieldOptions& a, ProtobufCFieldOptions& b) {
    a.Swap(&b);
  }
  inline void Swap(ProtobufCFieldOptions* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ProtobufCFieldOptions* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  ProtobufCFieldOptions* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<ProtobufCFieldOptions>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const ProtobufCFieldOptions& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom( const ProtobufCFieldOptions& from) {
    ProtobufCFieldOptions::MergeImpl(*this, from);
  }
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::PROTOBUF_NAMESPACE_ID::Arena* arena, bool is_message_owned);
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(ProtobufCFieldOptions* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "ProtobufCFieldOptions";
  }
  protected:
  explicit ProtobufCFieldOptions(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kStringAsBytesFieldNumber = 1,
  };
  // optional bool string_as_bytes = 1 [default = false];
  bool has_string_as_bytes() const;
  private:
  bool _internal_has_string_as_bytes() const;
  public:
  void clear_string_as_bytes();
  bool string_as_bytes() const;
  void set_string_as_bytes(bool value);
  private:
  bool _internal_string_as_bytes() const;
  void _internal_set_string_as_bytes(bool value);
  public:

  // @@protoc_insertion_point(class_scope:ProtobufCFieldOptions)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
    ::PROTOBUF_NAMESPACE_ID::internal::HasBits<1> _has_bits_;
    mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
    bool string_as_bytes_;
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_protobuf_2dc_2fprotobuf_2dc_2eproto;
};
// ===================================================================

static const int kPbCFileFieldNumber = 1019;
extern ::PROTOBUF_NAMESPACE_ID::internal::ExtensionIdentifier< ::PROTOBUF_NAMESPACE_ID::FileOptions,
    ::PROTOBUF_NAMESPACE_ID::internal::MessageTypeTraits< ::ProtobufCFileOptions >, 11, false >
  pb_c_file;
static const int kPbCMsgFieldNumber = 1019;
extern ::PROTOBUF_NAMESPACE_ID::internal::ExtensionIdentifier< ::PROTOBUF_NAMESPACE_ID::MessageOptions,
    ::PROTOBUF_NAMESPACE_ID::internal::MessageTypeTraits< ::ProtobufCMessageOptions >, 11, false >
  pb_c_msg;
static const int kPbCFieldFieldNumber = 1019;
extern ::PROTOBUF_NAMESPACE_ID::internal::ExtensionIdentifier< ::PROTOBUF_NAMESPACE_ID::FieldOptions,
    ::PROTOBUF_NAMESPACE_ID::internal::MessageTypeTraits< ::ProtobufCFieldOptions >, 11, false >
  pb_c_field;

// ===================================================================

#ifdef __GNUC__
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// ProtobufCFileOptions

// optional bool no_generate = 1 [default = false];
inline bool ProtobufCFileOptions::_internal_has_no_generate() const {
  bool value = (_impl_._has_bits_[0] & 0x00000002u) != 0;
  return value;
}
inline bool ProtobufCFileOptions::has_no_generate() const {
  return _internal_has_no_generate();
}
inline void ProtobufCFileOptions::clear_no_generate() {
  _impl_.no_generate_ = false;
  _impl_._has_bits_[0] &= ~0x00000002u;
}
inline bool ProtobufCFileOptions::_internal_no_generate() const {
  return _impl_.no_generate_;
}
inline bool ProtobufCFileOptions::no_generate() const {
  // @@protoc_insertion_point(field_get:ProtobufCFileOptions.no_generate)
  return _internal_no_generate();
}
inline void ProtobufCFileOptions::_internal_set_no_generate(bool value) {
  _impl_._has_bits_[0] |= 0x00000002u;
  _impl_.no_generate_ = value;
}
inline void ProtobufCFileOptions::set_no_generate(bool value) {
  _internal_set_no_generate(value);
  // @@protoc_insertion_point(field_set:ProtobufCFileOptions.no_generate)
}

// optional bool gen_pack_helpers = 2 [default = true];
inline bool ProtobufCFileOptions::_internal_has_gen_pack_helpers() const {
  bool value = (_impl_._has_bits_[0] & 0x00000010u) != 0;
  return value;
}
inline bool ProtobufCFileOptions::has_gen_pack_helpers() const {
  return _internal_has_gen_pack_helpers();
}
inline void ProtobufCFileOptions::clear_gen_pack_helpers() {
  _impl_.gen_pack_helpers_ = true;
  _impl_._has_bits_[0] &= ~0x00000010u;
}
inline bool ProtobufCFileOptions::_internal_gen_pack_helpers() const {
  return _impl_.gen_pack_helpers_;
}
inline bool ProtobufCFileOptions::gen_pack_helpers() const {
  // @@protoc_insertion_point(field_get:ProtobufCFileOptions.gen_pack_helpers)
  return _internal_gen_pack_helpers();
}
inline void ProtobufCFileOptions::_internal_set_gen_pack_helpers(bool value) {
  _impl_._has_bits_[0] |= 0x00000010u;
  _impl_.gen_pack_helpers_ = value;
}
inline void ProtobufCFileOptions::set_gen_pack_helpers(bool value) {
  _internal_set_gen_pack_helpers(value);
  // @@protoc_insertion_point(field_set:ProtobufCFileOptions.gen_pack_helpers)
}

// optional bool gen_init_helpers = 3 [default = true];
inline bool ProtobufCFileOptions::_internal_has_gen_init_helpers() const {
  bool value = (_impl_._has_bits_[0] & 0x00000020u) != 0;
  return value;
}
inline bool ProtobufCFileOptions::has_gen_init_helpers() const {
  return _internal_has_gen_init_helpers();
}
inline void ProtobufCFileOptions::clear_gen_init_helpers() {
  _impl_.gen_init_helpers_ = true;
  _impl_._has_bits_[0] &= ~0x00000020u;
}
inline bool ProtobufCFileOptions::_internal_gen_init_helpers() const {
  return _impl_.gen_init_helpers_;
}
inline bool ProtobufCFileOptions::gen_init_helpers() const {
  // @@protoc_insertion_point(field_get:ProtobufCFileOptions.gen_init_helpers)
  return _internal_gen_init_helpers();
}
inline void ProtobufCFileOptions::_internal_set_gen_init_helpers(bool value) {
  _impl_._has_bits_[0] |= 0x00000020u;
  _impl_.gen_init_helpers_ = value;
}
inline void ProtobufCFileOptions::set_gen_init_helpers(bool value) {
  _internal_set_gen_init_helpers(value);
  // @@protoc_insertion_point(field_set:ProtobufCFileOptions.gen_init_helpers)
}

// optional bool const_strings = 4 [default = false];
inline bool ProtobufCFileOptions::_internal_has_const_strings() const {
  bool value = (_impl_._has_bits_[0] & 0x00000004u) != 0;
  return value;
}
inline bool ProtobufCFileOptions::has_const_strings() const {
  return _internal_has_const_strings();
}
inline void ProtobufCFileOptions::clear_const_strings() {
  _impl_.const_strings_ = false;
  _impl_._has_bits_[0] &= ~0x00000004u;
}
inline bool ProtobufCFileOptions::_internal_const_strings() const {
  return _impl_.const_strings_;
}
inline bool ProtobufCFileOptions::const_strings() const {
  // @@protoc_insertion_point(field_get:ProtobufCFileOptions.const_strings)
  return _internal_const_strings();
}
inline void ProtobufCFileOptions::_internal_set_const_strings(bool value) {
  _impl_._has_bits_[0] |= 0x00000004u;
  _impl_.const_strings_ = value;
}
inline void ProtobufCFileOptions::set_const_strings(bool value) {
  _internal_set_const_strings(value);
  // @@protoc_insertion_point(field_set:ProtobufCFileOptions.const_strings)
}

// optional bool use_oneof_field_name = 5 [default = false];
inline bool ProtobufCFileOptions::_internal_has_use_oneof_field_name() const {
  bool value = (_impl_._has_bits_[0] & 0x00000008u) != 0;
  return value;
}
inline bool ProtobufCFileOptions::has_use_oneof_field_name() const {
  return _internal_has_use_oneof_field_name();
}
inline void ProtobufCFileOptions::clear_use_oneof_field_name() {
  _impl_.use_oneof_field_name_ = false;
  _impl_._has_bits_[0] &= ~0x00000008u;
}
inline bool ProtobufCFileOptions::_internal_use_oneof_field_name() const {
  return _impl_.use_oneof_field_name_;
}
inline bool ProtobufCFileOptions::use_oneof_field_name() const {
  // @@protoc_insertion_point(field_get:ProtobufCFileOptions.use_oneof_field_name)
  return _internal_use_oneof_field_name();
}
inline void ProtobufCFileOptions::_internal_set_use_oneof_field_name(bool value) {
  _impl_._has_bits_[0] |= 0x00000008u;
  _impl_.use_oneof_field_name_ = value;
}
inline void ProtobufCFileOptions::set_use_oneof_field_name(bool value) {
  _internal_set_use_oneof_field_name(value);
  // @@protoc_insertion_point(field_set:ProtobufCFileOptions.use_oneof_field_name)
}

// optional string c_package = 6;
inline bool ProtobufCFileOptions::_internal_has_c_package() const {
  bool value = (_impl_._has_bits_[0] & 0x00000001u) != 0;
  return value;
}
inline bool ProtobufCFileOptions::has_c_package() const {
  return _internal_has_c_package();
}
inline void ProtobufCFileOptions::clear_c_package() {
  _impl_.c_package_.ClearToEmpty();
  _impl_._has_bits_[0] &= ~0x00000001u;
}
inline const std::string& ProtobufCFileOptions::c_package() const {
  // @@protoc_insertion_point(field_get:ProtobufCFileOptions.c_package)
  return _internal_c_package();
}
template <typename ArgT0, typename... ArgT>
inline PROTOBUF_ALWAYS_INLINE
void ProtobufCFileOptions::set_c_package(ArgT0&& arg0, ArgT... args) {
 _impl_._has_bits_[0] |= 0x00000001u;
 _impl_.c_package_.Set(static_cast<ArgT0 &&>(arg0), args..., GetArenaForAllocation());
  // @@protoc_insertion_point(field_set:ProtobufCFileOptions.c_package)
}
inline std::string* ProtobufCFileOptions::mutable_c_package() {
  std::string* _s = _internal_mutable_c_package();
  // @@protoc_insertion_point(field_mutable:ProtobufCFileOptions.c_package)
  return _s;
}
inline const std::string& ProtobufCFileOptions::_internal_c_package() const {
  return _impl_.c_package_.Get();
}
inline void ProtobufCFileOptions::_internal_set_c_package(const std::string& value) {
  _impl_._has_bits_[0] |= 0x00000001u;
  _impl_.c_package_.Set(value, GetArenaForAllocation());
}
inline std::string* ProtobufCFileOptions::_internal_mutable_c_package() {
  _impl_._has_bits_[0] |= 0x00000001u;
  return _impl_.c_package_.Mutable(GetArenaForAllocation());
}
inline std::string* ProtobufCFileOptions::release_c_package() {
  // @@protoc_insertion_point(field_release:ProtobufCFileOptions.c_package)
  if (!_internal_has_c_package()) {
    return nullptr;
  }
  _impl_._has_bits_[0] &= ~0x00000001u;
  auto* p = _impl_.c_package_.Release();
#ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (_impl_.c_package_.IsDefault()) {
    _impl_.c_package_.Set("", GetArenaForAllocation());
  }
#endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  return p;
}
inline void ProtobufCFileOptions::set_allocated_c_package(std::string* c_package) {
  if (c_package != nullptr) {
    _impl_._has_bits_[0] |= 0x00000001u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000001u;
  }
  _impl_.c_package_.SetAllocated(c_package, GetArenaForAllocation());
#ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (_impl_.c_package_.IsDefault()) {
    _impl_.c_package_.Set("", GetArenaForAllocation());
  }
#endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  // @@protoc_insertion_point(field_set_allocated:ProtobufCFileOptions.c_package)
}

// -------------------------------------------------------------------

// ProtobufCMessageOptions

// optional bool gen_pack_helpers = 1 [default = false];
inline bool ProtobufCMessageOptions::_internal_has_gen_pack_helpers() const {
  bool value = (_impl_._has_bits_[0] & 0x00000002u) != 0;
  return value;
}
inline bool ProtobufCMessageOptions::has_gen_pack_helpers() const {
  return _internal_has_gen_pack_helpers();
}
inline void ProtobufCMessageOptions::clear_gen_pack_helpers() {
  _impl_.gen_pack_helpers_ = false;
  _impl_._has_bits_[0] &= ~0x00000002u;
}
inline bool ProtobufCMessageOptions::_internal_gen_pack_helpers() const {
  return _impl_.gen_pack_helpers_;
}
inline bool ProtobufCMessageOptions::gen_pack_helpers() const {
  // @@protoc_insertion_point(field_get:ProtobufCMessageOptions.gen_pack_helpers)
  return _internal_gen_pack_helpers();
}
inline void ProtobufCMessageOptions::_internal_set_gen_pack_helpers(bool value) {
  _impl_._has_bits_[0] |= 0x00000002u;
  _impl_.gen_pack_helpers_ = value;
}
inline void ProtobufCMessageOptions::set_gen_pack_helpers(bool value) {
  _internal_set_gen_pack_helpers(value);
  // @@protoc_insertion_point(field_set:ProtobufCMessageOptions.gen_pack_helpers)
}

// optional bool gen_init_helpers = 2 [default = true];
inline bool ProtobufCMessageOptions::_internal_has_gen_init_helpers() const {
  bool value = (_impl_._has_bits_[0] & 0x00000004u) != 0;
  return value;
}
inline bool ProtobufCMessageOptions::has_gen_init_helpers() const {
  return _internal_has_gen_init_helpers();
}
inline void ProtobufCMessageOptions::clear_gen_init_helpers() {
  _impl_.gen_init_helpers_ = true;
  _impl_._has_bits_[0] &= ~0x00000004u;
}
inline bool ProtobufCMessageOptions::_internal_gen_init_helpers() const {
  return _impl_.gen_init_helpers_;
}
inline bool ProtobufCMessageOptions::gen_init_helpers() const {
  // @@protoc_insertion_point(field_get:ProtobufCMessageOptions.gen_init_helpers)
  return _internal_gen_init_helpers();
}
inline void ProtobufCMessageOptions::_internal_set_gen_init_helpers(bool value) {
  _impl_._has_bits_[0] |= 0x00000004u;
  _impl_.gen_init_helpers_ = value;
}
inline void ProtobufCMessageOptions::set_gen_init_helpers(bool value) {
  _internal_set_gen_init_helpers(value);
  // @@protoc_insertion_point(field_set:ProtobufCMessageOptions.gen_init_helpers)
}

// optional string base_field_name = 3 [default = "base"];
inline bool ProtobufCMessageOptions::_internal_has_base_field_name() const {
  bool value = (_impl_._has_bits_[0] & 0x00000001u) != 0;
  return value;
}
inline bool ProtobufCMessageOptions::has_base_field_name() const {
  return _internal_has_base_field_name();
}
inline void ProtobufCMessageOptions::clear_base_field_name() {
  _impl_.base_field_name_.ClearToDefault(::ProtobufCMessageOptions::Impl_::_i_give_permission_to_break_this_code_default_base_field_name_, GetArenaForAllocation());
  _impl_._has_bits_[0] &= ~0x00000001u;
}
inline const std::string& ProtobufCMessageOptions::base_field_name() const {
  // @@protoc_insertion_point(field_get:ProtobufCMessageOptions.base_field_name)
  if (_impl_.base_field_name_.IsDefault()) return Impl_::_i_give_permission_to_break_this_code_default_base_field_name_.get();
  return _internal_base_field_name();
}
template <typename ArgT0, typename... ArgT>
inline PROTOBUF_ALWAYS_INLINE
void ProtobufCMessageOptions::set_base_field_name(ArgT0&& arg0, ArgT... args) {
 _impl_._has_bits_[0] |= 0x00000001u;
 _impl_.base_field_name_.Set(static_cast<ArgT0 &&>(arg0), args..., GetArenaForAllocation());
  // @@protoc_insertion_point(field_set:ProtobufCMessageOptions.base_field_name)
}
inline std::string* ProtobufCMessageOptions::mutable_base_field_name() {
  std::string* _s = _internal_mutable_base_field_name();
  // @@protoc_insertion_point(field_mutable:ProtobufCMessageOptions.base_field_name)
  return _s;
}
inline const std::string& ProtobufCMessageOptions::_internal_base_field_name() const {
  return _impl_.base_field_name_.Get();
}
inline void ProtobufCMessageOptions::_internal_set_base_field_name(const std::string& value) {
  _impl_._has_bits_[0] |= 0x00000001u;
  _impl_.base_field_name_.Set(value, GetArenaForAllocation());
}
inline std::string* ProtobufCMessageOptions::_internal_mutable_base_field_name() {
  _impl_._has_bits_[0] |= 0x00000001u;
  return _impl_.base_field_name_.Mutable(::ProtobufCMessageOptions::Impl_::_i_give_permission_to_break_this_code_default_base_field_name_, GetArenaForAllocation());
}
inline std::string* ProtobufCMessageOptions::release_base_field_name() {
  // @@protoc_insertion_point(field_release:ProtobufCMessageOptions.base_field_name)
  if (!_internal_has_base_field_name()) {
    return nullptr;
  }
  _impl_._has_bits_[0] &= ~0x00000001u;
  auto* p = _impl_.base_field_name_.Release();
  return p;
}
inline void ProtobufCMessageOptions::set_allocated_base_field_name(std::string* base_field_name) {
  if (base_field_name != nullptr) {
    _impl_._has_bits_[0] |= 0x00000001u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000001u;
  }
  _impl_.base_field_name_.SetAllocated(base_field_name, GetArenaForAllocation());
  // @@protoc_insertion_point(field_set_allocated:ProtobufCMessageOptions.base_field_name)
}

// -------------------------------------------------------------------

// ProtobufCFieldOptions

// optional bool string_as_bytes = 1 [default = false];
inline bool ProtobufCFieldOptions::_internal_has_string_as_bytes() const {
  bool value = (_impl_._has_bits_[0] & 0x00000001u) != 0;
  return value;
}
inline bool ProtobufCFieldOptions::has_string_as_bytes() const {
  return _internal_has_string_as_bytes();
}
inline void ProtobufCFieldOptions::clear_string_as_bytes() {
  _impl_.string_as_bytes_ = false;
  _impl_._has_bits_[0] &= ~0x00000001u;
}
inline bool ProtobufCFieldOptions::_internal_string_as_bytes() const {
  return _impl_.string_as_bytes_;
}
inline bool ProtobufCFieldOptions::string_as_bytes() const {
  // @@protoc_insertion_point(field_get:ProtobufCFieldOptions.string_as_bytes)
  return _internal_string_as_bytes();
}
inline void ProtobufCFieldOptions::_internal_set_string_as_bytes(bool value) {
  _impl_._has_bits_[0] |= 0x00000001u;
  _impl_.string_as_bytes_ = value;
}
inline void ProtobufCFieldOptions::set_string_as_bytes(bool value) {
  _internal_set_string_as_bytes(value);
  // @@protoc_insertion_point(field_set:ProtobufCFieldOptions.string_as_bytes)
}

#ifdef __GNUC__
  #pragma GCC diagnostic pop
#endif  // __GNUC__
// -------------------------------------------------------------------

// -------------------------------------------------------------------


// @@protoc_insertion_point(namespace_scope)


// @@protoc_insertion_point(global_scope)

#include <google/protobuf/port_undef.inc>
#endif  // GOOGLE_PROTOBUF_INCLUDED_GOOGLE_PROTOBUF_INCLUDED_protobuf_2dc_2fprotobuf_2dc_2eproto
