// Generated by the protocol buffer compiler.  DO NOT EDIT!
// source: protobuf-c/protobuf-c.proto

#include "protobuf-c/protobuf-c.pb.h"

#include <algorithm>

#include <google/protobuf/io/coded_stream.h>
#include <google/protobuf/extension_set.h>
#include <google/protobuf/wire_format_lite.h>
#include <google/protobuf/descriptor.h>
#include <google/protobuf/generated_message_reflection.h>
#include <google/protobuf/reflection_ops.h>
#include <google/protobuf/wire_format.h>
// @@protoc_insertion_point(includes)
#include <google/protobuf/port_def.inc>

PROTOBUF_PRAGMA_INIT_SEG

namespace _pb = ::PROTOBUF_NAMESPACE_ID;
namespace _pbi = _pb::internal;

PROTOBUF_CONSTEXPR ProtobufCFileOptions::ProtobufCFileOptions(
    ::_pbi::ConstantInitialized): _impl_{
    /*decltype(_impl_._has_bits_)*/{}
  , /*decltype(_impl_._cached_size_)*/{}
  , /*decltype(_impl_.c_package_)*/{&::_pbi::fixed_address_empty_string, ::_pbi::ConstantInitialized{}}
  , /*decltype(_impl_.no_generate_)*/false
  , /*decltype(_impl_.const_strings_)*/false
  , /*decltype(_impl_.use_oneof_field_name_)*/false
  , /*decltype(_impl_.gen_pack_helpers_)*/true
  , /*decltype(_impl_.gen_init_helpers_)*/true} {}
struct ProtobufCFileOptionsDefaultTypeInternal {
  PROTOBUF_CONSTEXPR ProtobufCFileOptionsDefaultTypeInternal()
      : _instance(::_pbi::ConstantInitialized{}) {}
  ~ProtobufCFileOptionsDefaultTypeInternal() {}
  union {
    ProtobufCFileOptions _instance;
  };
};
PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 ProtobufCFileOptionsDefaultTypeInternal _ProtobufCFileOptions_default_instance_;
PROTOBUF_CONSTEXPR ProtobufCMessageOptions::ProtobufCMessageOptions(
    ::_pbi::ConstantInitialized): _impl_{
    /*decltype(_impl_._has_bits_)*/{}
  , /*decltype(_impl_._cached_size_)*/{}
  , /*decltype(_impl_.base_field_name_)*/{nullptr, ::_pbi::ConstantInitialized{}}
  , /*decltype(_impl_.gen_pack_helpers_)*/false
  , /*decltype(_impl_.gen_init_helpers_)*/true} {}
struct ProtobufCMessageOptionsDefaultTypeInternal {
  PROTOBUF_CONSTEXPR ProtobufCMessageOptionsDefaultTypeInternal()
      : _instance(::_pbi::ConstantInitialized{}) {}
  ~ProtobufCMessageOptionsDefaultTypeInternal() {}
  union {
    ProtobufCMessageOptions _instance;
  };
};
PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 ProtobufCMessageOptionsDefaultTypeInternal _ProtobufCMessageOptions_default_instance_;
PROTOBUF_CONSTEXPR ProtobufCFieldOptions::ProtobufCFieldOptions(
    ::_pbi::ConstantInitialized): _impl_{
    /*decltype(_impl_._has_bits_)*/{}
  , /*decltype(_impl_._cached_size_)*/{}
  , /*decltype(_impl_.string_as_bytes_)*/false} {}
struct ProtobufCFieldOptionsDefaultTypeInternal {
  PROTOBUF_CONSTEXPR ProtobufCFieldOptionsDefaultTypeInternal()
      : _instance(::_pbi::ConstantInitialized{}) {}
  ~ProtobufCFieldOptionsDefaultTypeInternal() {}
  union {
    ProtobufCFieldOptions _instance;
  };
};
PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 ProtobufCFieldOptionsDefaultTypeInternal _ProtobufCFieldOptions_default_instance_;
static ::_pb::Metadata file_level_metadata_protobuf_2dc_2fprotobuf_2dc_2eproto[3];
static constexpr ::_pb::EnumDescriptor const** file_level_enum_descriptors_protobuf_2dc_2fprotobuf_2dc_2eproto = nullptr;
static constexpr ::_pb::ServiceDescriptor const** file_level_service_descriptors_protobuf_2dc_2fprotobuf_2dc_2eproto = nullptr;

const uint32_t TableStruct_protobuf_2dc_2fprotobuf_2dc_2eproto::offsets[] PROTOBUF_SECTION_VARIABLE(protodesc_cold) = {
  PROTOBUF_FIELD_OFFSET(::ProtobufCFileOptions, _impl_._has_bits_),
  PROTOBUF_FIELD_OFFSET(::ProtobufCFileOptions, _internal_metadata_),
  ~0u,  // no _extensions_
  ~0u,  // no _oneof_case_
  ~0u,  // no _weak_field_map_
  ~0u,  // no _inlined_string_donated_
  PROTOBUF_FIELD_OFFSET(::ProtobufCFileOptions, _impl_.no_generate_),
  PROTOBUF_FIELD_OFFSET(::ProtobufCFileOptions, _impl_.gen_pack_helpers_),
  PROTOBUF_FIELD_OFFSET(::ProtobufCFileOptions, _impl_.gen_init_helpers_),
  PROTOBUF_FIELD_OFFSET(::ProtobufCFileOptions, _impl_.const_strings_),
  PROTOBUF_FIELD_OFFSET(::ProtobufCFileOptions, _impl_.use_oneof_field_name_),
  PROTOBUF_FIELD_OFFSET(::ProtobufCFileOptions, _impl_.c_package_),
  1,
  4,
  5,
  2,
  3,
  0,
  PROTOBUF_FIELD_OFFSET(::ProtobufCMessageOptions, _impl_._has_bits_),
  PROTOBUF_FIELD_OFFSET(::ProtobufCMessageOptions, _internal_metadata_),
  ~0u,  // no _extensions_
  ~0u,  // no _oneof_case_
  ~0u,  // no _weak_field_map_
  ~0u,  // no _inlined_string_donated_
  PROTOBUF_FIELD_OFFSET(::ProtobufCMessageOptions, _impl_.gen_pack_helpers_),
  PROTOBUF_FIELD_OFFSET(::ProtobufCMessageOptions, _impl_.gen_init_helpers_),
  PROTOBUF_FIELD_OFFSET(::ProtobufCMessageOptions, _impl_.base_field_name_),
  1,
  2,
  0,
  PROTOBUF_FIELD_OFFSET(::ProtobufCFieldOptions, _impl_._has_bits_),
  PROTOBUF_FIELD_OFFSET(::ProtobufCFieldOptions, _internal_metadata_),
  ~0u,  // no _extensions_
  ~0u,  // no _oneof_case_
  ~0u,  // no _weak_field_map_
  ~0u,  // no _inlined_string_donated_
  PROTOBUF_FIELD_OFFSET(::ProtobufCFieldOptions, _impl_.string_as_bytes_),
  0,
};
static const ::_pbi::MigrationSchema schemas[] PROTOBUF_SECTION_VARIABLE(protodesc_cold) = {
  { 0, 12, -1, sizeof(::ProtobufCFileOptions)},
  { 18, 27, -1, sizeof(::ProtobufCMessageOptions)},
  { 30, 37, -1, sizeof(::ProtobufCFieldOptions)},
};

static const ::_pb::Message* const file_default_instances[] = {
  &::_ProtobufCFileOptions_default_instance_._instance,
  &::_ProtobufCMessageOptions_default_instance_._instance,
  &::_ProtobufCFieldOptions_default_instance_._instance,
};

const char descriptor_table_protodef_protobuf_2dc_2fprotobuf_2dc_2eproto[] PROTOBUF_SECTION_VARIABLE(protodesc_cold) =
  "\n\033protobuf-c/protobuf-c.proto\032 google/pr"
  "otobuf/descriptor.proto\"\310\001\n\024ProtobufCFil"
  "eOptions\022\032\n\013no_generate\030\001 \001(\010:\005false\022\036\n\020"
  "gen_pack_helpers\030\002 \001(\010:\004true\022\036\n\020gen_init"
  "_helpers\030\003 \001(\010:\004true\022\034\n\rconst_strings\030\004 "
  "\001(\010:\005false\022#\n\024use_oneof_field_name\030\005 \001(\010"
  ":\005false\022\021\n\tc_package\030\006 \001(\t\"y\n\027ProtobufCM"
  "essageOptions\022\037\n\020gen_pack_helpers\030\001 \001(\010:"
  "\005false\022\036\n\020gen_init_helpers\030\002 \001(\010:\004true\022\035"
  "\n\017base_field_name\030\003 \001(\t:\004base\"7\n\025Protobu"
  "fCFieldOptions\022\036\n\017string_as_bytes\030\001 \001(\010:"
  "\005false:G\n\tpb_c_file\022\034.google.protobuf.Fi"
  "leOptions\030\373\007 \001(\0132\025.ProtobufCFileOptions:"
  "L\n\010pb_c_msg\022\037.google.protobuf.MessageOpt"
  "ions\030\373\007 \001(\0132\030.ProtobufCMessageOptions:J\n"
  "\npb_c_field\022\035.google.protobuf.FieldOptio"
  "ns\030\373\007 \001(\0132\026.ProtobufCFieldOptionsB\005\332\?\002\010\001"
  ;
static const ::_pbi::DescriptorTable* const descriptor_table_protobuf_2dc_2fprotobuf_2dc_2eproto_deps[1] = {
  &::descriptor_table_google_2fprotobuf_2fdescriptor_2eproto,
};
static ::_pbi::once_flag descriptor_table_protobuf_2dc_2fprotobuf_2dc_2eproto_once;
const ::_pbi::DescriptorTable descriptor_table_protobuf_2dc_2fprotobuf_2dc_2eproto = {
    false, false, 680, descriptor_table_protodef_protobuf_2dc_2fprotobuf_2dc_2eproto,
    "protobuf-c/protobuf-c.proto",
    &descriptor_table_protobuf_2dc_2fprotobuf_2dc_2eproto_once, descriptor_table_protobuf_2dc_2fprotobuf_2dc_2eproto_deps, 1, 3,
    schemas, file_default_instances, TableStruct_protobuf_2dc_2fprotobuf_2dc_2eproto::offsets,
    file_level_metadata_protobuf_2dc_2fprotobuf_2dc_2eproto, file_level_enum_descriptors_protobuf_2dc_2fprotobuf_2dc_2eproto,
    file_level_service_descriptors_protobuf_2dc_2fprotobuf_2dc_2eproto,
};
PROTOBUF_ATTRIBUTE_WEAK const ::_pbi::DescriptorTable* descriptor_table_protobuf_2dc_2fprotobuf_2dc_2eproto_getter() {
  return &descriptor_table_protobuf_2dc_2fprotobuf_2dc_2eproto;
}

// Force running AddDescriptors() at dynamic initialization time.
PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 static ::_pbi::AddDescriptorsRunner dynamic_init_dummy_protobuf_2dc_2fprotobuf_2dc_2eproto(&descriptor_table_protobuf_2dc_2fprotobuf_2dc_2eproto);

// ===================================================================

class ProtobufCFileOptions::_Internal {
 public:
  using HasBits = decltype(std::declval<ProtobufCFileOptions>()._impl_._has_bits_);
  static void set_has_no_generate(HasBits* has_bits) {
    (*has_bits)[0] |= 2u;
  }
  static void set_has_gen_pack_helpers(HasBits* has_bits) {
    (*has_bits)[0] |= 16u;
  }
  static void set_has_gen_init_helpers(HasBits* has_bits) {
    (*has_bits)[0] |= 32u;
  }
  static void set_has_const_strings(HasBits* has_bits) {
    (*has_bits)[0] |= 4u;
  }
  static void set_has_use_oneof_field_name(HasBits* has_bits) {
    (*has_bits)[0] |= 8u;
  }
  static void set_has_c_package(HasBits* has_bits) {
    (*has_bits)[0] |= 1u;
  }
};

ProtobufCFileOptions::ProtobufCFileOptions(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                         bool is_message_owned)
  : ::PROTOBUF_NAMESPACE_ID::Message(arena, is_message_owned) {
  SharedCtor(arena, is_message_owned);
  // @@protoc_insertion_point(arena_constructor:ProtobufCFileOptions)
}
ProtobufCFileOptions::ProtobufCFileOptions(const ProtobufCFileOptions& from)
  : ::PROTOBUF_NAMESPACE_ID::Message() {
  ProtobufCFileOptions* const _this = this; (void)_this;
  new (&_impl_) Impl_{
      decltype(_impl_._has_bits_){from._impl_._has_bits_}
    , /*decltype(_impl_._cached_size_)*/{}
    , decltype(_impl_.c_package_){}
    , decltype(_impl_.no_generate_){}
    , decltype(_impl_.const_strings_){}
    , decltype(_impl_.use_oneof_field_name_){}
    , decltype(_impl_.gen_pack_helpers_){}
    , decltype(_impl_.gen_init_helpers_){}};

  _internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
  _impl_.c_package_.InitDefault();
  #ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
    _impl_.c_package_.Set("", GetArenaForAllocation());
  #endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (from._internal_has_c_package()) {
    _this->_impl_.c_package_.Set(from._internal_c_package(), 
      _this->GetArenaForAllocation());
  }
  ::memcpy(&_impl_.no_generate_, &from._impl_.no_generate_,
    static_cast<size_t>(reinterpret_cast<char*>(&_impl_.gen_init_helpers_) -
    reinterpret_cast<char*>(&_impl_.no_generate_)) + sizeof(_impl_.gen_init_helpers_));
  // @@protoc_insertion_point(copy_constructor:ProtobufCFileOptions)
}

inline void ProtobufCFileOptions::SharedCtor(
    ::_pb::Arena* arena, bool is_message_owned) {
  (void)arena;
  (void)is_message_owned;
  new (&_impl_) Impl_{
      decltype(_impl_._has_bits_){}
    , /*decltype(_impl_._cached_size_)*/{}
    , decltype(_impl_.c_package_){}
    , decltype(_impl_.no_generate_){false}
    , decltype(_impl_.const_strings_){false}
    , decltype(_impl_.use_oneof_field_name_){false}
    , decltype(_impl_.gen_pack_helpers_){true}
    , decltype(_impl_.gen_init_helpers_){true}
  };
  _impl_.c_package_.InitDefault();
  #ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
    _impl_.c_package_.Set("", GetArenaForAllocation());
  #endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
}

ProtobufCFileOptions::~ProtobufCFileOptions() {
  // @@protoc_insertion_point(destructor:ProtobufCFileOptions)
  if (auto *arena = _internal_metadata_.DeleteReturnArena<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>()) {
  (void)arena;
    return;
  }
  SharedDtor();
}

inline void ProtobufCFileOptions::SharedDtor() {
  GOOGLE_DCHECK(GetArenaForAllocation() == nullptr);
  _impl_.c_package_.Destroy();
}

void ProtobufCFileOptions::SetCachedSize(int size) const {
  _impl_._cached_size_.Set(size);
}

void ProtobufCFileOptions::Clear() {
// @@protoc_insertion_point(message_clear_start:ProtobufCFileOptions)
  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (cached_has_bits & 0x00000001u) {
    _impl_.c_package_.ClearNonDefaultToEmpty();
  }
  ::memset(&_impl_.no_generate_, 0, static_cast<size_t>(
      reinterpret_cast<char*>(&_impl_.use_oneof_field_name_) -
      reinterpret_cast<char*>(&_impl_.no_generate_)) + sizeof(_impl_.use_oneof_field_name_));
  if (cached_has_bits & 0x00000030u) {
    _impl_.gen_pack_helpers_ = true;
    _impl_.gen_init_helpers_ = true;
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
}

const char* ProtobufCFileOptions::_InternalParse(const char* ptr, ::_pbi::ParseContext* ctx) {
#define CHK_(x) if (PROTOBUF_PREDICT_FALSE(!(x))) goto failure
  _Internal::HasBits has_bits{};
  while (!ctx->Done(&ptr)) {
    uint32_t tag;
    ptr = ::_pbi::ReadTag(ptr, &tag);
    switch (tag >> 3) {
      // optional bool no_generate = 1 [default = false];
      case 1:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 8)) {
          _Internal::set_has_no_generate(&has_bits);
          _impl_.no_generate_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint64(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // optional bool gen_pack_helpers = 2 [default = true];
      case 2:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 16)) {
          _Internal::set_has_gen_pack_helpers(&has_bits);
          _impl_.gen_pack_helpers_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint64(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // optional bool gen_init_helpers = 3 [default = true];
      case 3:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 24)) {
          _Internal::set_has_gen_init_helpers(&has_bits);
          _impl_.gen_init_helpers_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint64(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // optional bool const_strings = 4 [default = false];
      case 4:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 32)) {
          _Internal::set_has_const_strings(&has_bits);
          _impl_.const_strings_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint64(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // optional bool use_oneof_field_name = 5 [default = false];
      case 5:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 40)) {
          _Internal::set_has_use_oneof_field_name(&has_bits);
          _impl_.use_oneof_field_name_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint64(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // optional string c_package = 6;
      case 6:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 50)) {
          auto str = _internal_mutable_c_package();
          ptr = ::_pbi::InlineGreedyStringParser(str, ptr, ctx);
          CHK_(ptr);
          #ifndef NDEBUG
          ::_pbi::VerifyUTF8(str, "ProtobufCFileOptions.c_package");
          #endif  // !NDEBUG
        } else
          goto handle_unusual;
        continue;
      default:
        goto handle_unusual;
    }  // switch
  handle_unusual:
    if ((tag == 0) || ((tag & 7) == 4)) {
      CHK_(ptr);
      ctx->SetLastTag(tag);
      goto message_done;
    }
    ptr = UnknownFieldParse(
        tag,
        _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(),
        ptr, ctx);
    CHK_(ptr != nullptr);
  }  // while
message_done:
  _impl_._has_bits_.Or(has_bits);
  return ptr;
failure:
  ptr = nullptr;
  goto message_done;
#undef CHK_
}

uint8_t* ProtobufCFileOptions::_InternalSerialize(
    uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const {
  // @@protoc_insertion_point(serialize_to_array_start:ProtobufCFileOptions)
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  // optional bool no_generate = 1 [default = false];
  if (cached_has_bits & 0x00000002u) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteBoolToArray(1, this->_internal_no_generate(), target);
  }

  // optional bool gen_pack_helpers = 2 [default = true];
  if (cached_has_bits & 0x00000010u) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteBoolToArray(2, this->_internal_gen_pack_helpers(), target);
  }

  // optional bool gen_init_helpers = 3 [default = true];
  if (cached_has_bits & 0x00000020u) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteBoolToArray(3, this->_internal_gen_init_helpers(), target);
  }

  // optional bool const_strings = 4 [default = false];
  if (cached_has_bits & 0x00000004u) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteBoolToArray(4, this->_internal_const_strings(), target);
  }

  // optional bool use_oneof_field_name = 5 [default = false];
  if (cached_has_bits & 0x00000008u) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteBoolToArray(5, this->_internal_use_oneof_field_name(), target);
  }

  // optional string c_package = 6;
  if (cached_has_bits & 0x00000001u) {
    ::PROTOBUF_NAMESPACE_ID::internal::WireFormat::VerifyUTF8StringNamedField(
      this->_internal_c_package().data(), static_cast<int>(this->_internal_c_package().length()),
      ::PROTOBUF_NAMESPACE_ID::internal::WireFormat::SERIALIZE,
      "ProtobufCFileOptions.c_package");
    target = stream->WriteStringMaybeAliased(
        6, this->_internal_c_package(), target);
  }

  if (PROTOBUF_PREDICT_FALSE(_internal_metadata_.have_unknown_fields())) {
    target = ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
        _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:ProtobufCFileOptions)
  return target;
}

size_t ProtobufCFileOptions::ByteSizeLong() const {
// @@protoc_insertion_point(message_byte_size_start:ProtobufCFileOptions)
  size_t total_size = 0;

  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (cached_has_bits & 0x0000003fu) {
    // optional string c_package = 6;
    if (cached_has_bits & 0x00000001u) {
      total_size += 1 +
        ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::StringSize(
          this->_internal_c_package());
    }

    // optional bool no_generate = 1 [default = false];
    if (cached_has_bits & 0x00000002u) {
      total_size += 1 + 1;
    }

    // optional bool const_strings = 4 [default = false];
    if (cached_has_bits & 0x00000004u) {
      total_size += 1 + 1;
    }

    // optional bool use_oneof_field_name = 5 [default = false];
    if (cached_has_bits & 0x00000008u) {
      total_size += 1 + 1;
    }

    // optional bool gen_pack_helpers = 2 [default = true];
    if (cached_has_bits & 0x00000010u) {
      total_size += 1 + 1;
    }

    // optional bool gen_init_helpers = 3 [default = true];
    if (cached_has_bits & 0x00000020u) {
      total_size += 1 + 1;
    }

  }
  return MaybeComputeUnknownFieldsSize(total_size, &_impl_._cached_size_);
}

const ::PROTOBUF_NAMESPACE_ID::Message::ClassData ProtobufCFileOptions::_class_data_ = {
    ::PROTOBUF_NAMESPACE_ID::Message::CopyWithSourceCheck,
    ProtobufCFileOptions::MergeImpl
};
const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*ProtobufCFileOptions::GetClassData() const { return &_class_data_; }


void ProtobufCFileOptions::MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg) {
  auto* const _this = static_cast<ProtobufCFileOptions*>(&to_msg);
  auto& from = static_cast<const ProtobufCFileOptions&>(from_msg);
  // @@protoc_insertion_point(class_specific_merge_from_start:ProtobufCFileOptions)
  GOOGLE_DCHECK_NE(&from, _this);
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (cached_has_bits & 0x0000003fu) {
    if (cached_has_bits & 0x00000001u) {
      _this->_internal_set_c_package(from._internal_c_package());
    }
    if (cached_has_bits & 0x00000002u) {
      _this->_impl_.no_generate_ = from._impl_.no_generate_;
    }
    if (cached_has_bits & 0x00000004u) {
      _this->_impl_.const_strings_ = from._impl_.const_strings_;
    }
    if (cached_has_bits & 0x00000008u) {
      _this->_impl_.use_oneof_field_name_ = from._impl_.use_oneof_field_name_;
    }
    if (cached_has_bits & 0x00000010u) {
      _this->_impl_.gen_pack_helpers_ = from._impl_.gen_pack_helpers_;
    }
    if (cached_has_bits & 0x00000020u) {
      _this->_impl_.gen_init_helpers_ = from._impl_.gen_init_helpers_;
    }
    _this->_impl_._has_bits_[0] |= cached_has_bits;
  }
  _this->_internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
}

void ProtobufCFileOptions::CopyFrom(const ProtobufCFileOptions& from) {
// @@protoc_insertion_point(class_specific_copy_from_start:ProtobufCFileOptions)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}

bool ProtobufCFileOptions::IsInitialized() const {
  return true;
}

void ProtobufCFileOptions::InternalSwap(ProtobufCFileOptions* other) {
  using std::swap;
  auto* lhs_arena = GetArenaForAllocation();
  auto* rhs_arena = other->GetArenaForAllocation();
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr::InternalSwap(
      &_impl_.c_package_, lhs_arena,
      &other->_impl_.c_package_, rhs_arena
  );
  ::PROTOBUF_NAMESPACE_ID::internal::memswap<
      PROTOBUF_FIELD_OFFSET(ProtobufCFileOptions, _impl_.use_oneof_field_name_)
      + sizeof(ProtobufCFileOptions::_impl_.use_oneof_field_name_)
      - PROTOBUF_FIELD_OFFSET(ProtobufCFileOptions, _impl_.no_generate_)>(
          reinterpret_cast<char*>(&_impl_.no_generate_),
          reinterpret_cast<char*>(&other->_impl_.no_generate_));
  swap(_impl_.gen_pack_helpers_, other->_impl_.gen_pack_helpers_);
  swap(_impl_.gen_init_helpers_, other->_impl_.gen_init_helpers_);
}

::PROTOBUF_NAMESPACE_ID::Metadata ProtobufCFileOptions::GetMetadata() const {
  return ::_pbi::AssignDescriptors(
      &descriptor_table_protobuf_2dc_2fprotobuf_2dc_2eproto_getter, &descriptor_table_protobuf_2dc_2fprotobuf_2dc_2eproto_once,
      file_level_metadata_protobuf_2dc_2fprotobuf_2dc_2eproto[0]);
}

// ===================================================================

class ProtobufCMessageOptions::_Internal {
 public:
  using HasBits = decltype(std::declval<ProtobufCMessageOptions>()._impl_._has_bits_);
  static void set_has_gen_pack_helpers(HasBits* has_bits) {
    (*has_bits)[0] |= 2u;
  }
  static void set_has_gen_init_helpers(HasBits* has_bits) {
    (*has_bits)[0] |= 4u;
  }
  static void set_has_base_field_name(HasBits* has_bits) {
    (*has_bits)[0] |= 1u;
  }
};

const ::PROTOBUF_NAMESPACE_ID::internal::LazyString ProtobufCMessageOptions::Impl_::_i_give_permission_to_break_this_code_default_base_field_name_{{{"base", 4}}, {nullptr}};
ProtobufCMessageOptions::ProtobufCMessageOptions(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                         bool is_message_owned)
  : ::PROTOBUF_NAMESPACE_ID::Message(arena, is_message_owned) {
  SharedCtor(arena, is_message_owned);
  // @@protoc_insertion_point(arena_constructor:ProtobufCMessageOptions)
}
ProtobufCMessageOptions::ProtobufCMessageOptions(const ProtobufCMessageOptions& from)
  : ::PROTOBUF_NAMESPACE_ID::Message() {
  ProtobufCMessageOptions* const _this = this; (void)_this;
  new (&_impl_) Impl_{
      decltype(_impl_._has_bits_){from._impl_._has_bits_}
    , /*decltype(_impl_._cached_size_)*/{}
    , decltype(_impl_.base_field_name_){}
    , decltype(_impl_.gen_pack_helpers_){}
    , decltype(_impl_.gen_init_helpers_){}};

  _internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
  _impl_.base_field_name_.InitDefault();
  if (from._internal_has_base_field_name()) {
    _this->_impl_.base_field_name_.Set(from._internal_base_field_name(), 
      _this->GetArenaForAllocation());
  }
  ::memcpy(&_impl_.gen_pack_helpers_, &from._impl_.gen_pack_helpers_,
    static_cast<size_t>(reinterpret_cast<char*>(&_impl_.gen_init_helpers_) -
    reinterpret_cast<char*>(&_impl_.gen_pack_helpers_)) + sizeof(_impl_.gen_init_helpers_));
  // @@protoc_insertion_point(copy_constructor:ProtobufCMessageOptions)
}

inline void ProtobufCMessageOptions::SharedCtor(
    ::_pb::Arena* arena, bool is_message_owned) {
  (void)arena;
  (void)is_message_owned;
  new (&_impl_) Impl_{
      decltype(_impl_._has_bits_){}
    , /*decltype(_impl_._cached_size_)*/{}
    , decltype(_impl_.base_field_name_){}
    , decltype(_impl_.gen_pack_helpers_){false}
    , decltype(_impl_.gen_init_helpers_){true}
  };
  _impl_.base_field_name_.InitDefault();
}

ProtobufCMessageOptions::~ProtobufCMessageOptions() {
  // @@protoc_insertion_point(destructor:ProtobufCMessageOptions)
  if (auto *arena = _internal_metadata_.DeleteReturnArena<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>()) {
  (void)arena;
    return;
  }
  SharedDtor();
}

inline void ProtobufCMessageOptions::SharedDtor() {
  GOOGLE_DCHECK(GetArenaForAllocation() == nullptr);
  _impl_.base_field_name_.Destroy();
}

void ProtobufCMessageOptions::SetCachedSize(int size) const {
  _impl_._cached_size_.Set(size);
}

void ProtobufCMessageOptions::Clear() {
// @@protoc_insertion_point(message_clear_start:ProtobufCMessageOptions)
  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (cached_has_bits & 0x00000001u) {
    _impl_.base_field_name_.ClearToDefault(::ProtobufCMessageOptions::Impl_::_i_give_permission_to_break_this_code_default_base_field_name_, GetArenaForAllocation());
     }
  if (cached_has_bits & 0x00000006u) {
    _impl_.gen_pack_helpers_ = false;
    _impl_.gen_init_helpers_ = true;
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
}

const char* ProtobufCMessageOptions::_InternalParse(const char* ptr, ::_pbi::ParseContext* ctx) {
#define CHK_(x) if (PROTOBUF_PREDICT_FALSE(!(x))) goto failure
  _Internal::HasBits has_bits{};
  while (!ctx->Done(&ptr)) {
    uint32_t tag;
    ptr = ::_pbi::ReadTag(ptr, &tag);
    switch (tag >> 3) {
      // optional bool gen_pack_helpers = 1 [default = false];
      case 1:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 8)) {
          _Internal::set_has_gen_pack_helpers(&has_bits);
          _impl_.gen_pack_helpers_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint64(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // optional bool gen_init_helpers = 2 [default = true];
      case 2:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 16)) {
          _Internal::set_has_gen_init_helpers(&has_bits);
          _impl_.gen_init_helpers_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint64(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // optional string base_field_name = 3 [default = "base"];
      case 3:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 26)) {
          auto str = _internal_mutable_base_field_name();
          ptr = ::_pbi::InlineGreedyStringParser(str, ptr, ctx);
          CHK_(ptr);
          #ifndef NDEBUG
          ::_pbi::VerifyUTF8(str, "ProtobufCMessageOptions.base_field_name");
          #endif  // !NDEBUG
        } else
          goto handle_unusual;
        continue;
      default:
        goto handle_unusual;
    }  // switch
  handle_unusual:
    if ((tag == 0) || ((tag & 7) == 4)) {
      CHK_(ptr);
      ctx->SetLastTag(tag);
      goto message_done;
    }
    ptr = UnknownFieldParse(
        tag,
        _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(),
        ptr, ctx);
    CHK_(ptr != nullptr);
  }  // while
message_done:
  _impl_._has_bits_.Or(has_bits);
  return ptr;
failure:
  ptr = nullptr;
  goto message_done;
#undef CHK_
}

uint8_t* ProtobufCMessageOptions::_InternalSerialize(
    uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const {
  // @@protoc_insertion_point(serialize_to_array_start:ProtobufCMessageOptions)
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  // optional bool gen_pack_helpers = 1 [default = false];
  if (cached_has_bits & 0x00000002u) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteBoolToArray(1, this->_internal_gen_pack_helpers(), target);
  }

  // optional bool gen_init_helpers = 2 [default = true];
  if (cached_has_bits & 0x00000004u) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteBoolToArray(2, this->_internal_gen_init_helpers(), target);
  }

  // optional string base_field_name = 3 [default = "base"];
  if (cached_has_bits & 0x00000001u) {
    ::PROTOBUF_NAMESPACE_ID::internal::WireFormat::VerifyUTF8StringNamedField(
      this->_internal_base_field_name().data(), static_cast<int>(this->_internal_base_field_name().length()),
      ::PROTOBUF_NAMESPACE_ID::internal::WireFormat::SERIALIZE,
      "ProtobufCMessageOptions.base_field_name");
    target = stream->WriteStringMaybeAliased(
        3, this->_internal_base_field_name(), target);
  }

  if (PROTOBUF_PREDICT_FALSE(_internal_metadata_.have_unknown_fields())) {
    target = ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
        _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:ProtobufCMessageOptions)
  return target;
}

size_t ProtobufCMessageOptions::ByteSizeLong() const {
// @@protoc_insertion_point(message_byte_size_start:ProtobufCMessageOptions)
  size_t total_size = 0;

  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (cached_has_bits & 0x00000007u) {
    // optional string base_field_name = 3 [default = "base"];
    if (cached_has_bits & 0x00000001u) {
      total_size += 1 +
        ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::StringSize(
          this->_internal_base_field_name());
    }

    // optional bool gen_pack_helpers = 1 [default = false];
    if (cached_has_bits & 0x00000002u) {
      total_size += 1 + 1;
    }

    // optional bool gen_init_helpers = 2 [default = true];
    if (cached_has_bits & 0x00000004u) {
      total_size += 1 + 1;
    }

  }
  return MaybeComputeUnknownFieldsSize(total_size, &_impl_._cached_size_);
}

const ::PROTOBUF_NAMESPACE_ID::Message::ClassData ProtobufCMessageOptions::_class_data_ = {
    ::PROTOBUF_NAMESPACE_ID::Message::CopyWithSourceCheck,
    ProtobufCMessageOptions::MergeImpl
};
const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*ProtobufCMessageOptions::GetClassData() const { return &_class_data_; }


void ProtobufCMessageOptions::MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg) {
  auto* const _this = static_cast<ProtobufCMessageOptions*>(&to_msg);
  auto& from = static_cast<const ProtobufCMessageOptions&>(from_msg);
  // @@protoc_insertion_point(class_specific_merge_from_start:ProtobufCMessageOptions)
  GOOGLE_DCHECK_NE(&from, _this);
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (cached_has_bits & 0x00000007u) {
    if (cached_has_bits & 0x00000001u) {
      _this->_internal_set_base_field_name(from._internal_base_field_name());
    }
    if (cached_has_bits & 0x00000002u) {
      _this->_impl_.gen_pack_helpers_ = from._impl_.gen_pack_helpers_;
    }
    if (cached_has_bits & 0x00000004u) {
      _this->_impl_.gen_init_helpers_ = from._impl_.gen_init_helpers_;
    }
    _this->_impl_._has_bits_[0] |= cached_has_bits;
  }
  _this->_internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
}

void ProtobufCMessageOptions::CopyFrom(const ProtobufCMessageOptions& from) {
// @@protoc_insertion_point(class_specific_copy_from_start:ProtobufCMessageOptions)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}

bool ProtobufCMessageOptions::IsInitialized() const {
  return true;
}

void ProtobufCMessageOptions::InternalSwap(ProtobufCMessageOptions* other) {
  using std::swap;
  auto* lhs_arena = GetArenaForAllocation();
  auto* rhs_arena = other->GetArenaForAllocation();
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr::InternalSwap(
      &_impl_.base_field_name_, lhs_arena,
      &other->_impl_.base_field_name_, rhs_arena
  );
  swap(_impl_.gen_pack_helpers_, other->_impl_.gen_pack_helpers_);
  swap(_impl_.gen_init_helpers_, other->_impl_.gen_init_helpers_);
}

::PROTOBUF_NAMESPACE_ID::Metadata ProtobufCMessageOptions::GetMetadata() const {
  return ::_pbi::AssignDescriptors(
      &descriptor_table_protobuf_2dc_2fprotobuf_2dc_2eproto_getter, &descriptor_table_protobuf_2dc_2fprotobuf_2dc_2eproto_once,
      file_level_metadata_protobuf_2dc_2fprotobuf_2dc_2eproto[1]);
}

// ===================================================================

class ProtobufCFieldOptions::_Internal {
 public:
  using HasBits = decltype(std::declval<ProtobufCFieldOptions>()._impl_._has_bits_);
  static void set_has_string_as_bytes(HasBits* has_bits) {
    (*has_bits)[0] |= 1u;
  }
};

ProtobufCFieldOptions::ProtobufCFieldOptions(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                         bool is_message_owned)
  : ::PROTOBUF_NAMESPACE_ID::Message(arena, is_message_owned) {
  SharedCtor(arena, is_message_owned);
  // @@protoc_insertion_point(arena_constructor:ProtobufCFieldOptions)
}
ProtobufCFieldOptions::ProtobufCFieldOptions(const ProtobufCFieldOptions& from)
  : ::PROTOBUF_NAMESPACE_ID::Message() {
  ProtobufCFieldOptions* const _this = this; (void)_this;
  new (&_impl_) Impl_{
      decltype(_impl_._has_bits_){from._impl_._has_bits_}
    , /*decltype(_impl_._cached_size_)*/{}
    , decltype(_impl_.string_as_bytes_){}};

  _internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
  _this->_impl_.string_as_bytes_ = from._impl_.string_as_bytes_;
  // @@protoc_insertion_point(copy_constructor:ProtobufCFieldOptions)
}

inline void ProtobufCFieldOptions::SharedCtor(
    ::_pb::Arena* arena, bool is_message_owned) {
  (void)arena;
  (void)is_message_owned;
  new (&_impl_) Impl_{
      decltype(_impl_._has_bits_){}
    , /*decltype(_impl_._cached_size_)*/{}
    , decltype(_impl_.string_as_bytes_){false}
  };
}

ProtobufCFieldOptions::~ProtobufCFieldOptions() {
  // @@protoc_insertion_point(destructor:ProtobufCFieldOptions)
  if (auto *arena = _internal_metadata_.DeleteReturnArena<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>()) {
  (void)arena;
    return;
  }
  SharedDtor();
}

inline void ProtobufCFieldOptions::SharedDtor() {
  GOOGLE_DCHECK(GetArenaForAllocation() == nullptr);
}

void ProtobufCFieldOptions::SetCachedSize(int size) const {
  _impl_._cached_size_.Set(size);
}

void ProtobufCFieldOptions::Clear() {
// @@protoc_insertion_point(message_clear_start:ProtobufCFieldOptions)
  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  _impl_.string_as_bytes_ = false;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
}

const char* ProtobufCFieldOptions::_InternalParse(const char* ptr, ::_pbi::ParseContext* ctx) {
#define CHK_(x) if (PROTOBUF_PREDICT_FALSE(!(x))) goto failure
  _Internal::HasBits has_bits{};
  while (!ctx->Done(&ptr)) {
    uint32_t tag;
    ptr = ::_pbi::ReadTag(ptr, &tag);
    switch (tag >> 3) {
      // optional bool string_as_bytes = 1 [default = false];
      case 1:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 8)) {
          _Internal::set_has_string_as_bytes(&has_bits);
          _impl_.string_as_bytes_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint64(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      default:
        goto handle_unusual;
    }  // switch
  handle_unusual:
    if ((tag == 0) || ((tag & 7) == 4)) {
      CHK_(ptr);
      ctx->SetLastTag(tag);
      goto message_done;
    }
    ptr = UnknownFieldParse(
        tag,
        _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(),
        ptr, ctx);
    CHK_(ptr != nullptr);
  }  // while
message_done:
  _impl_._has_bits_.Or(has_bits);
  return ptr;
failure:
  ptr = nullptr;
  goto message_done;
#undef CHK_
}

uint8_t* ProtobufCFieldOptions::_InternalSerialize(
    uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const {
  // @@protoc_insertion_point(serialize_to_array_start:ProtobufCFieldOptions)
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  // optional bool string_as_bytes = 1 [default = false];
  if (cached_has_bits & 0x00000001u) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteBoolToArray(1, this->_internal_string_as_bytes(), target);
  }

  if (PROTOBUF_PREDICT_FALSE(_internal_metadata_.have_unknown_fields())) {
    target = ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
        _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:ProtobufCFieldOptions)
  return target;
}

size_t ProtobufCFieldOptions::ByteSizeLong() const {
// @@protoc_insertion_point(message_byte_size_start:ProtobufCFieldOptions)
  size_t total_size = 0;

  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  // optional bool string_as_bytes = 1 [default = false];
  cached_has_bits = _impl_._has_bits_[0];
  if (cached_has_bits & 0x00000001u) {
    total_size += 1 + 1;
  }

  return MaybeComputeUnknownFieldsSize(total_size, &_impl_._cached_size_);
}

const ::PROTOBUF_NAMESPACE_ID::Message::ClassData ProtobufCFieldOptions::_class_data_ = {
    ::PROTOBUF_NAMESPACE_ID::Message::CopyWithSourceCheck,
    ProtobufCFieldOptions::MergeImpl
};
const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*ProtobufCFieldOptions::GetClassData() const { return &_class_data_; }


void ProtobufCFieldOptions::MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg) {
  auto* const _this = static_cast<ProtobufCFieldOptions*>(&to_msg);
  auto& from = static_cast<const ProtobufCFieldOptions&>(from_msg);
  // @@protoc_insertion_point(class_specific_merge_from_start:ProtobufCFieldOptions)
  GOOGLE_DCHECK_NE(&from, _this);
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  if (from._internal_has_string_as_bytes()) {
    _this->_internal_set_string_as_bytes(from._internal_string_as_bytes());
  }
  _this->_internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
}

void ProtobufCFieldOptions::CopyFrom(const ProtobufCFieldOptions& from) {
// @@protoc_insertion_point(class_specific_copy_from_start:ProtobufCFieldOptions)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}

bool ProtobufCFieldOptions::IsInitialized() const {
  return true;
}

void ProtobufCFieldOptions::InternalSwap(ProtobufCFieldOptions* other) {
  using std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  swap(_impl_.string_as_bytes_, other->_impl_.string_as_bytes_);
}

::PROTOBUF_NAMESPACE_ID::Metadata ProtobufCFieldOptions::GetMetadata() const {
  return ::_pbi::AssignDescriptors(
      &descriptor_table_protobuf_2dc_2fprotobuf_2dc_2eproto_getter, &descriptor_table_protobuf_2dc_2fprotobuf_2dc_2eproto_once,
      file_level_metadata_protobuf_2dc_2fprotobuf_2dc_2eproto[2]);
}
PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 ::PROTOBUF_NAMESPACE_ID::internal::ExtensionIdentifier< ::PROTOBUF_NAMESPACE_ID::FileOptions,
    ::PROTOBUF_NAMESPACE_ID::internal::MessageTypeTraits< ::ProtobufCFileOptions >, 11, false>
  pb_c_file(kPbCFileFieldNumber, ::ProtobufCFileOptions::default_instance(), nullptr);
PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 ::PROTOBUF_NAMESPACE_ID::internal::ExtensionIdentifier< ::PROTOBUF_NAMESPACE_ID::MessageOptions,
    ::PROTOBUF_NAMESPACE_ID::internal::MessageTypeTraits< ::ProtobufCMessageOptions >, 11, false>
  pb_c_msg(kPbCMsgFieldNumber, ::ProtobufCMessageOptions::default_instance(), nullptr);
PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 ::PROTOBUF_NAMESPACE_ID::internal::ExtensionIdentifier< ::PROTOBUF_NAMESPACE_ID::FieldOptions,
    ::PROTOBUF_NAMESPACE_ID::internal::MessageTypeTraits< ::ProtobufCFieldOptions >, 11, false>
  pb_c_field(kPbCFieldFieldNumber, ::ProtobufCFieldOptions::default_instance(), nullptr);

// @@protoc_insertion_point(namespace_scope)
PROTOBUF_NAMESPACE_OPEN
template<> PROTOBUF_NOINLINE ::ProtobufCFileOptions*
Arena::CreateMaybeMessage< ::ProtobufCFileOptions >(Arena* arena) {
  return Arena::CreateMessageInternal< ::ProtobufCFileOptions >(arena);
}
template<> PROTOBUF_NOINLINE ::ProtobufCMessageOptions*
Arena::CreateMaybeMessage< ::ProtobufCMessageOptions >(Arena* arena) {
  return Arena::CreateMessageInternal< ::ProtobufCMessageOptions >(arena);
}
template<> PROTOBUF_NOINLINE ::ProtobufCFieldOptions*
Arena::CreateMaybeMessage< ::ProtobufCFieldOptions >(Arena* arena) {
  return Arena::CreateMessageInternal< ::ProtobufCFieldOptions >(arena);
}
PROTOBUF_NAMESPACE_CLOSE

// @@protoc_insertion_point(global_scope)
#include <google/protobuf/port_undef.inc>
